CON
  _CLKMODE      = XTAL1 + PLL16X                        
  _CLKFREQ      = 80_000_000
  _byteLimit = 100 

OBJ
  pst    : "Parallax Serial Terminal"
  co     : "Crank"
  
  fcrank : "jm_freqin"
  n : "Numbers" 

VAR
  long tmp

  byte b_delimiter, b_prefix[_byteLimit], b_suffix[_byteLimit]
    byte b_waitingToBeParsed[_byteLimit]
  byte b_byteMoveIndex
  byte b_RxString[_byteLimit]
  
  long l_timeStamp, l_CrankPeriod, l_pstReadCrankWheel, l_crankFreq, l_updateRateDenominator, l_toothGapRpm, l_toothRpm, {
  } l_calculatedCamRpm, l_timeStampSubtracted, l_CntPeak, l_sendAllValuesOnce, l_myReceivedByte, {
  } l_previousNum6, l_previousNum5, l_previousNum4, l_previousNum3, l_previousNum2, l_current, l_timeStampCrank, {
  }l_previous5average, l_Cnt, l_timeStampCrankPrevious, l_sendAllValues
    
  long comStack[100]
  
PUB Main
  pst.Start(115_200)

  ' wait until any key has been pushed. This way you can switch from Propeller Tool
  ' to PST without a hurry. At least during development all my terminal-based programs
  ' use this loop.
     
    tmp:=-1
    repeat while tmp==-1
      pst.str( string( "Crank",$0d,"== please press key to start ==",$0d ) )
      tmp:=pst.rxcheck
      if tmp==-1
         waitcnt( clkfreq+cnt )
     
    pst.Clear
    pst.str( string( "Starting Crank COG!",$0d ) )
     
    co.start(4, 36, 2, 6000)
     
    pst.str( string("frequency set to ") )
    pst.dec( co.getFrequency )

    cognew(com, @comStack)     

    fcrank.init(5)
    l_pstReadCrankWheel := 1
    b_delimiter := "="

    l_timeStamp := cnt
  
  repeat


    l_CrankPeriod := fcrank.period
    if(l_CrankPeriod) <> 0 and l_pstReadCrankWheel == 1
      calculateCrankRpm(l_CrankPeriod)
        
    l_CrankPeriod := fcrank.freq
    
    l_crankFreq := (((l_CrankPeriod*60)/10)/36)





    
PUB com | timeStamp


  l_updateRateDenominator := 2
   
  timeStamp := cnt

  repeat

    if((cnt-timeStamp) > (clkfreq/l_updateRateDenominator))
    
      timeStamp := cnt
      pst.Dec(l_toothGapRpm)
      pst.char(13)
      pst.Dec(l_toothRpm)
      pst.char(13)
      pst.dec(l_timeStampSubtracted)
      pst.char(13)
      pst.dec(l_CntPeak)
      pst.char(13)
      pst.dec(fcrank.period)
      pst.char(13)
      pst.dec(((clkfreq/fcrank.period)*60)/36)
      pst.char(13)
      pst.dec(l_crankFreq)
      pst.char(13)
      pst.str(@b_waitingToBeParsed)
      pst.char(13)
      pst.char(13)  
         
   
    if (l_sendAllValuesOnce == 1)
      if(cnt > (timeStamp + (clkfreq/l_updateRateDenominator))) 
        sendValues
        l_sendAllValuesOnce := 0
   
    if (l_sendAllValues == 1)
      sendValues 
           
   
   
    l_myReceivedByte := pst.RxCheck
     
    if l_myReceivedByte <> -1
      b_waitingToBeParsed[b_byteMoveIndex++] := l_myReceivedByte
     
      pst.str(@b_waitingToBeParsed)
      pst.str(string(pst#NL))
     
      if l_myReceivedByte == 13
        DelimiterFinder(@b_waitingToBeParsed)
        ByteFill(@b_waitingToBeParsed, 0, strsize(@b_waitingToBeParsed))
        b_byteMoveIndex := 0    
  


PUB calculateCrankRpm(localCrankPeriod)


    l_previousNum6 := l_previousNum5
    l_previousNum5 := l_previousNum4
    l_previousNum4 := l_previousNum3
    l_previousNum3 := l_previousNum2
    l_previousNum2 := l_current
    l_current := localCrankPeriod  

    if (l_timeStampCrank == 0)
      l_timeStampCrank := (cnt)
      
    if (l_previousNum6 <> 0 and l_previousNum5 <> 0 and l_previousNum4 <> 0 and l_previousNum3 <> 0 and l_previousNum2 <> 0)
      l_previous5average := ((l_previousNum6 + l_previousNum5 + l_previousNum4 + l_previousNum3 + l_previousNum2)/5)
 
      if (((l_current/10)*100) > ((l_previous5average/10)* 110)) 'or ((current*100) < (previous5average * 90))

          l_Cnt += 1    
            if((cnt-l_timeStampCrank) > clkfreq)
              l_timeStampCrankPrevious := l_timeStampCrank                  
              l_timeStampCrank := cnt
              l_timeStampSubtracted := (l_timeStampCrank-l_timeStampCrankPrevious)
              l_CntPeak := l_Cnt ' I should a number here matching see rpm/60, for a frequency of 6000 I should see 166.67 of these each second 
              l_Cnt := 0

          l_current := l_previous5average        ' DO NOT COMMENT OUT
                     
      else         
        
        
        if (((clkfreq/l_current)*60)/36) < 90
            ' error            
            
        else
            l_toothRpm := (((clkfreq/l_previous5average)*60)/36)


PUB DelimiterFinder(RxStringAddr) : idx | localCount, localIndex, pstDelimiterFinder                                            

  ' Initialize variables here
  localIndex := 0
  pstDelimiterFinder := 0

  repeat
    ifnot localCount := byte[RxStringAddr][idx++]                ' ifnot c, itterate to the next byte in byte[RxStringAddr] ?
      pst.Str(String("No delimiter, aborted"))
      pst.str(string(pst#NL))
      return -1

    pst.char(localCount)
    pst.str(string(pst#NL))  
  until localCount == b_delimiter'localCount == "="
 
  bytemove(@b_prefix, RxStringAddr, --idx)              ' idx points to the character after the delimiter, copy prefix
  b_prefix[idx] := 0                                    ' add terminator  
  RxStringAddr += idx + 1                               ' advance pointer, skip delimiter (+1)                                                 
  bytemove(@b_suffix, RxStringAddr, strsize(RxStringAddr) +1)                   ' copy tail including the existing terminator (+1)
  variableUpdator(@b_prefix, @b_suffix)




PUB variableUpdator(preAddr, sufAddr) | localIndex, pstVariableUpdator   ' variableUpdator(@prefix, @suffix)

  ' Initialize variables here
  localIndex := 0
  pstVariableUpdator := 0

  if strcomp(preAddr, string("updateRateDenominator")) 'strcomp(@prefix, var1)
    
    l_updateRateDenominator := n.FromStr(sufAddr, %000_000_000_0_0_000000_01010) ' n.FromStr(@suffix, %000_000_000_0_0_000000_01010)    ' converts from a string to a dec
    pst.str(l_updateRateDenominator)
    pst.str(string(pst#NL))

  if strcomp(preAddr, string("sendAllValuesOnce")) 'strcomp(@prefix, var1)
    
    l_sendAllValuesOnce := n.FromStr(sufAddr, %000_000_000_0_0_000000_01010) ' n.FromStr(@suffix, %000_000_000_0_0_000000_01010)    ' converts from a string to a dec
    pst.str(l_sendAllValuesOnce)
    pst.str(string(pst#NL))

  if strcomp(preAddr, string("sendAllValues")) 'strcomp(@prefix, var1)
    
    l_sendAllValues := n.FromStr(sufAddr, %000_000_000_0_0_000000_01010) ' n.FromStr(@suffix, %000_000_000_0_0_000000_01010)    ' converts from a string to a dec
    pst.str(l_sendAllValues)
    pst.str(string(pst#NL))    
    
  {if strcomp(preAddr, l_i1Name) 'strcomp(@prefix, var1)
    
    l_i1value := n.FromStr(sufAddr, %000_000_000_0_0_000000_01010) ' n.FromStr(@suffix, %000_000_000_0_0_000000_01010)    ' converts from a string to a dec


    long sendAllValuesOnce
  long sendAllValues



  
     }


PUB sendValues | localIndex, pstSendAllValues

  ' Initialize variables here
  localIndex := 0
  pstSendAllValues := 0 

  pst.Str(String("sendAllValues()", pst#NL))        